/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * vim: sw=2 ts=2 sts=2 expandtab 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
interface mozIStorageRow;

[scriptable, uuid(18dd7953-076d-4598-8105-3e32ad26ab24)]
interface mozIStorageResultSet : nsISupports {

  /**
   * Obtains the next row from the result set from the statement that was
   * executed.
   *
   * @returns the next row from the result set.  This will be null when there
   *          are no more results.
   */
  mozIStorageRow getNextRow();
};
